/***********************************************************/
/*               Star Simulation - by Sjeep                */
/***********************************************************/
/*                                                         */
/* No, this isnt a mind-blowing amazing looking demo - its */
/* something nice and simple, aimed at helping newbies to  */
/* the ps2dev scene get started. This is distributed as is.*/
/* Do with it as you want, try and learn from it, and dont */
/* try and blame me for problems this code may cause.      */
/*                                                         */
/* A BIG thanks goes to Vzzrzzn. Most of the DMA/GS/IRQ    */
/* code (in hw.s) is his work. Thanks also goes to Gustavo */
/* Scotti for writing psx2lib, and Andrew Kieschnick for   */
/* Naplink - without the hard work of these people, this   */
/* simple little program would not have been possible.     */
/*                                                         */
/* Btw, this doesnt look good on a PAL console (because    */
/* of the big gap at the bottom of the screen). This is    */
/* because Vzzrzzn's display code is made for a NTSC       */
/* console. I'm sure you'll get over it :)                 */
/*                                                         */
/***********************************************************/
/*          (c) Nick Van Veen (a.k.a Sjeep) 2001           */
/*  e-mail: nickvv@xtra.co.nz               ICQ: 28057146  */
/*                                                         */
/*  Changes by Frank Buss (aka Shine) 2004:                */
/*  - compiles now with the new build environment          */
/*  - changed all DMA transfers to GIF_FIFO                */
/*  - font integration                                     */
/*  - screenshot integration                               */
/*  - removing star cluster in center by fading to black   */
/*  - code cleanups                                        */
/*  e-mail: fb@frank-buss.de                               */
/***********************************************************/

// #define SCREENSHOTS 1

#include <tamtypes.h>
#include <sifcmd.h>
#include <kernel.h>
#include <sifrpc.h>
#include <string.h>
#include <fileio.h>
#include "sio.h"
#include "regs.h"

#include "hershey.h"

#include "screenshot.h"
#include "gs.h"

// Defines related to the starfield
#define SF_SIZE		1000
#define Z_MAX  		100
#define SPEED  		2

#define Z_MAX5		(Z_MAX - 50)
#define Z_MAX4		(Z_MAX - 32)
#define Z_MAX3		(Z_MAX - 24)
#define Z_MAX2		(Z_MAX - 16)
#define Z_MAX1		(Z_MAX - 8)
#define Z_MAX0		(Z_MAX - 4)


typedef struct {
	long x, y, z;
} t_starfield;

t_starfield starfield[SF_SIZE];

void drawStars();
void drawText();
void createStar(int i);
void initStars();

// nice looking random algorithm
int rand(void)
{
	static int seed = 1;
	seed = (161140751 * seed + 13) % 219441163;
	return seed;
}


int main()
{
	#ifdef SCREENSHOTS
	int i = 0;
	#endif

	SifInitRpc(0); 
	nprintf("\nStar Simulation - By Sjeep\n");
	nprintf("\nVector Font Output - By Shine\n");

	fioInit();
	
	// init video
	initGS();

	initStars();

	while(1) {
		setColor(0, 0, 0);
		fillRect(0, 0, g_width, g_height);
		drawStars();  // update starfield
		drawText();

		#ifdef SCREENSHOTS
		if (i >= 100 && i < 140 && (i % 4) == 0) {
			char* name = "host:starsim0.tga";
			name[12] = (i - 100) / 4 + '0';
			ps2_screenshot_file(name, 0, g_width, g_height, PS2SS_GSPSMCT32);
			i++;
		} else i++;
		#endif

		waitVSync();  // wait twice, because it is interlaced
		waitVSync();
		flip();
	}
}

void drawStars()
{
	int x, y, i;

	for (i = 0; i < SF_SIZE; i++)
	{
		starfield[i].z -= SPEED;
		if(starfield[i].z < 2) createStar(i);
		x = (g_width/2) + ((starfield[i].x<<6) / starfield[i].z);
		y = (g_height/2) + ((starfield[i].y<<6) / starfield[i].z);

		if (x < 0 || y < 0 || x >= g_width || y >= g_height) {
			createStar(i);
			x = g_width / 2 + starfield[i].x / starfield[i].z;
			y = g_height / 2 + starfield[i].y / starfield[i].z;
		}

		// Choose star colour, based on the Z value. Gives fading-in effect.
		int color;
		if (starfield[i].z > Z_MAX0) color = 0;
		else if(starfield[i].z > Z_MAX2) color = 32;
		else if(starfield[i].z > Z_MAX2) color = 64;
		else if(starfield[i].z > Z_MAX3) color = 128;
		else if(starfield[i].z > Z_MAX4) color = 192; else color = 255;

		setColor(color, color, color);
		fillRect(x, y, x+2, y+2);
	}
}

void drawText()
{
	static int c = -50;
	static float scale = 2.1f;
	int y = ((float) (c * c + 1000)) / 17.0f;
	int width;
	char* text = "Stars and Text";
	if (c++ == 50) c = -50;
	
	setColor(0, 255, 255);
	width = (int) hersheyGetWidth(text, scale);
	hersheyDrawString(text, (g_width - width) / 2, y, scale, drawLine);
	hersheyDrawString(text, (g_width - width) / 2 + 1, y, scale, drawLine);
	hersheyDrawString(text, (g_width - width) / 2, y + 1, scale, drawLine);
	hersheyDrawString(text, (g_width - width) / 2 + 1, y + 1, scale, drawLine);
	
	setColor(255, 255, 255);
	hersheyDrawString("Star Simulation by Sjeep 2001", 10, g_height - 80, 0.8f , drawLine);
	hersheyDrawString("Vector Font Output by Shine 2004", 10, g_height - 40, 0.8f, drawLine);
}

void createStar(int i)
{
	// Set a star to new random values
	starfield[i].x = rand()%g_width;
	starfield[i].y = rand()%g_height;
	starfield[i].z = Z_MAX;
}

void initStars()
{
	int i;
	for (i = 0; i < SF_SIZE; i++) createStar(i);
}
